@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,76 +42,92 @@
 
 #include <stdio.h>
 #include "_scm.h"
+#include "stackchk.h"
+#ifdef DEBUG_EXTENSIONS
+#include "debug.h"
+#endif
+#include "dynwind.h"
 
+#include "continuations.h"
 
 
 /* {Continuations}
  */
 
-SCM scm_throwval = SCM_UNDEFINED;
-
 static char s_cont[] = "continuation";
 
-#ifdef __STDC__
+
 SCM 
-scm_make_cont (void)
-#else
-SCM 
-scm_make_cont ()
-#endif
+scm_make_cont (answer)
+     SCM * answer;
 {
   long j;
   SCM cont;
+
 #ifdef CHEAP_CONTINUATIONS
-  NEWCELL (cont);
-  DEFER_INTS;
-  SETJMPBUF (cont, scm_must_malloc ((long) sizeof (regs), s_cont));
-  CAR (cont) = tc7_contin;
-  DYNENV (cont) = dynwinds;
-  BASE (cont) = BASE (rootcont);
-  SEQ (cont) = SEQ (rootcont);
-  ALLOW_INTS;
+  SCM_NEWCELL (cont);
+  *answer = cont;
+  SCM_DEFER_INTS;
+  SCM_SETJMPBUF (cont, scm_must_malloc ((long) sizeof (scm_contregs), s_cont));
+  SCM_SETCAR (cont, scm_tc7_contin);
+  SCM_DYNENV (cont) = scm_dynwinds;
+  SCM_THROW_VALUE = SCM_EOL;
+  SCM_BASE (cont) = SCM_BASE (rootcont);
+  SCM_SEQ (cont) = SCM_SEQ (rootcont);
+  SCM_ALLOW_INTS;
 #else
-  register STACKITEM *src, *dst;
-  NEWCELL (cont);
-  DEFER_INTS;
-  FLUSH_REGISTER_WINDOWS;
-  j = scm_stack_size (BASE (rootcont));
-  SETJMPBUF (cont,
-	     scm_must_malloc ((long) (sizeof (regs) + j * sizeof (STACKITEM)),
+  register SCM_STACKITEM *src, *dst;
+
+#if 0
+  {
+    SCM winds;
+
+    for (winds = scm_dynwinds; winds != SCM_EOL; winds = SCM_CDR (winds))
+      {
+	if (SCM_INUMP (SCM_CAR (winds)))
+	  {
+	    scm_relocate_chunk_to_heap (SCM_CAR (winds));
+	  }
+      }
+  }
+#endif
+
+  SCM_NEWCELL (cont);
+  *answer = cont;
+  SCM_DEFER_INTS;
+  SCM_FLUSH_REGISTER_WINDOWS;
+  j = scm_stack_size (SCM_BASE (scm_rootcont));
+  SCM_SETJMPBUF (cont,
+	     scm_must_malloc ((long) (sizeof (scm_contregs) + j * sizeof (SCM_STACKITEM)),
 			      s_cont));
-  SETLENGTH (cont, j, tc7_contin);
-  DYNENV (cont) = dynwinds;
-  src = BASE (cont) = BASE (rootcont);
-  SEQ (cont) = SEQ (rootcont);
-  ALLOW_INTS;
-#ifndef STACK_GROWS_UP
-  src -= LENGTH (cont);
-#endif /* ndef STACK_GROWS_UP */
-  dst = (STACKITEM *) (CHARS (cont) + sizeof (regs));
-  for (j = LENGTH (cont); 0 <= --j;)
+  SCM_SETLENGTH (cont, j, scm_tc7_contin);
+  SCM_DYNENV (cont) = scm_dynwinds;
+  SCM_THROW_VALUE (cont) = SCM_EOL;
+  src = SCM_BASE (cont) = SCM_BASE (scm_rootcont);
+  SCM_SEQ (cont) = SCM_SEQ (scm_rootcont);
+  SCM_ALLOW_INTS;
+#ifndef SCM_STACK_GROWS_UP
+  src -= SCM_LENGTH (cont);
+#endif /* ndef SCM_STACK_GROWS_UP */
+  dst = (SCM_STACKITEM *) (SCM_CHARS (cont) + sizeof (scm_contregs));
+  for (j = SCM_LENGTH (cont); 0 <= --j;)
     *dst++ = *src++;
 #endif /* def CHEAP_CONTINUATIONS */
 #ifdef DEBUG_EXTENSIONS
-  DFRAME (cont) = last_debug_info_frame;
+  SCM_DFRAME (cont) = scm_last_debug_frame;
 #endif
   return cont;
 }
 
-
-void scm_dynthrow P ((SCM *a));
-
 /* Grow the stack so that there is room */
 /* to copy in the continuation.  Then */
 #ifndef CHEAP_CONTINUATIONS
-#ifdef __STDC__
-static void 
-grow_throw (SCM *a)
-#else
+
+static void grow_throw SCM_P ((SCM *a));
+
 static void 
 grow_throw (a)
      SCM *a;
-#endif
 {				/* retry the throw. */
   SCM growth[100];
   growth[0] = a[0];
@@ -122,20 +138,16 @@ grow_throw (a)
 }
 #endif /* ndef CHEAP_CONTINUATIONS */
 
-#ifdef __STDC__
-void 
-scm_dynthrow (SCM *a)
-#else
+
 void 
 scm_dynthrow (a)
      SCM *a;
-#endif
 {
   SCM cont = a[0], val = a[1];
 #ifndef CHEAP_CONTINUATIONS
   register long j;
-  register STACKITEM *src, *dst = BASE (rootcont);
-#ifdef STACK_GROWS_UP
+  register SCM_STACKITEM *src, *dst = SCM_BASE (scm_rootcont);
+#ifdef SCM_STACK_GROWS_UP
   if (a[2] && (a - ((SCM *) a[3]) < 100))
 #else
   if (a[2] && (((SCM *) a[3]) - a < 100))
@@ -144,64 +156,56 @@ scm_dynthrow (a)
 	   stderr);
   /* if (a[2]) fprintf(stderr, " ct = %ld, dist = %ld\n",
 			  a[2], (((SCM *)a[3]) - a)); */
-#ifdef STACK_GROWS_UP
-  if (PTR_GE (dst + LENGTH (cont), (STACKITEM *) & a))
+#ifdef SCM_STACK_GROWS_UP
+  if (SCM_PTR_GE (dst + SCM_LENGTH (cont), (SCM_STACKITEM *) & a))
     grow_throw (a);
 #else
-  dst -= LENGTH (cont);
-  if (PTR_LE (dst, (STACKITEM *) & a))
+  dst -= SCM_LENGTH (cont);
+  if (SCM_PTR_LE (dst, (SCM_STACKITEM *) & a))
     grow_throw (a);
-#endif /* def STACK_GROWS_UP */
-  FLUSH_REGISTER_WINDOWS;
-  src = (STACKITEM *) (CHARS (cont) + sizeof (regs));
-  for (j = LENGTH (cont); 0 <= --j;)
+#endif /* def SCM_STACK_GROWS_UP */
+  SCM_FLUSH_REGISTER_WINDOWS;
+  src = (SCM_STACKITEM *) (SCM_CHARS (cont) + sizeof (scm_contregs));
+  for (j = SCM_LENGTH (cont); 0 <= --j;)
     *dst++ = *src++;
 #ifdef sparc			/* clear out stack up to this stackframe */
   /* maybe this would help, maybe not */
-/*	bzero((void *)&a, sizeof(STACKITEM) * (((STACKITEM *)&a) -
-					       (dst - LENGTH(cont)))) */
+/*	bzero((void *)&a, sizeof(SCM_STACKITEM) * (((SCM_STACKITEM *)&a) -
+					       (dst - SCM_LENGTH(cont)))) */
 #endif
 #endif /* ndef CHEAP_CONTINUATIONS */
 #ifdef DEBUG_EXTENSIONS
-  last_debug_info_frame = DFRAME (cont);
+  scm_last_debug_frame = SCM_DFRAME (cont);
 #endif
-  scm_throwval = val;
-  longjmp (JMPBUF (cont), 1);
+  SCM_THROW_VALUE(cont) = val;
+  longjmp (SCM_JMPBUF (cont), 1);
 }
 
-#ifdef __STDC__
+
 SCM
-scm_throw (SCM cont, SCM val)
-#else
-SCM
-scm_throw (cont, val)
+scm_call_continuation (cont, val)
      SCM cont;
      SCM val;
-#endif
 {
   SCM a[3];
   a[0] = cont;
   a[1] = val;
   a[2] = 0;
-  if (   (SEQ (cont) != SEQ (rootcont))
-      || (BASE (cont) != BASE (rootcont)))  /* base compare not needed */
+  if (   (SCM_SEQ (cont) != SCM_SEQ (scm_rootcont))
+      || (SCM_BASE (cont) != SCM_BASE (scm_rootcont)))  /* base compare not needed */
     scm_wta (cont, "continuation from wrong top level", s_cont);
   
-  scm_dowinds (DYNENV (cont),
-	       scm_ilength (dynwinds) - scm_ilength (DYNENV (cont)));
+  scm_dowinds (SCM_DYNENV (cont),
+	       scm_ilength (scm_dynwinds) - scm_ilength (SCM_DYNENV (cont)));
   
   scm_dynthrow (a);
-  return UNSPECIFIED; /* not reached */
+  return SCM_UNSPECIFIED; /* not reached */
 }
 
 
-#ifdef __STDC__
-void
-scm_init_continuations (void)
-#else
+
 void
 scm_init_continuations ()
-#endif
 {
 #include "continuations.x"
 }
