@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,7 +42,10 @@
 
 #include <stdio.h>
 #include "_scm.h"
+#include "eval.h"
+#include "alist.h"
 
+#include "dynwind.h"
 
 
 /* {Dynamic wind}
@@ -50,70 +53,86 @@
 
 
 
-PROC (s_dynamic_wind, "dynamic-wind", 3, 0, 0, scm_dynamic_wind);
-#ifdef __STDC__
-SCM 
-scm_dynamic_wind (SCM thunk1, SCM thunk2, SCM thunk3)
-#else
+SCM_PROC(s_dynamic_wind, "dynamic-wind", 3, 0, 0, scm_dynamic_wind);
+
 SCM 
 scm_dynamic_wind (thunk1, thunk2, thunk3)
      SCM thunk1;
      SCM thunk2;
      SCM thunk3;
-#endif
 {
   SCM ans;
-  scm_apply (thunk1, EOL, EOL);
-  dynwinds = scm_acons (thunk1, thunk3, dynwinds);
-  ans = scm_apply (thunk2, EOL, EOL);
-  dynwinds = CDR (dynwinds);
-  scm_apply (thunk3, EOL, EOL);
+  scm_apply (thunk1, SCM_EOL, SCM_EOL);
+  scm_dynwinds = scm_acons (thunk1, thunk3, scm_dynwinds);
+  ans = scm_apply (thunk2, SCM_EOL, SCM_EOL);
+  scm_dynwinds = SCM_CDR (scm_dynwinds);
+  scm_apply (thunk3, SCM_EOL, SCM_EOL);
   return ans;
 }
 
-#ifdef __STDC__
-void 
-scm_dowinds (SCM to, long delta)
-#else
+
 void 
 scm_dowinds (to, delta)
      SCM to;
      long delta;
-#endif
 {
  tail:
-  if (dynwinds == to);
+  if (scm_dynwinds == to);
   else if (0 > delta)
     {
+      SCM wind_elt;
       SCM wind_key;
-      scm_dowinds (CDR (to), 1 + delta);
-      wind_key = CAR (CAR (to));
-      if (!(NIMP (wind_key) && SYMBOLP (wind_key)) && (wind_key != BOOL_F) && (wind_key != BOOL_T))
-	scm_apply (wind_key, EOL, EOL);
-      dynwinds = to;
+
+      scm_dowinds (SCM_CDR (to), 1 + delta);
+      wind_elt = SCM_CAR (to);
+#if 0
+      if (SCM_INUMP (wind_elt))
+	{
+	  scm_cross_dynwind_binding_scope (wind_elt, 0);
+	}
+      else
+#endif
+	{
+	  wind_key = SCM_CAR (wind_elt);
+	  if (   !(SCM_NIMP (wind_key) && SCM_SYMBOLP (wind_key))
+	      && (wind_key != SCM_BOOL_F)
+	      && (wind_key != SCM_BOOL_T))
+	    scm_apply (wind_key, SCM_EOL, SCM_EOL);
+	}
+      scm_dynwinds = to;
     }
   else
     {
       SCM from;
+      SCM wind_elt;
       SCM wind_key;
-      from = CDR (CAR (dynwinds));
-      wind_key = CAR (CAR (dynwinds));
-      dynwinds = CDR (dynwinds);
-      if (!(NIMP (wind_key) && SYMBOLP (wind_key)) && (wind_key != BOOL_F) && (wind_key != BOOL_T))
-	scm_apply (from, EOL, EOL);
+
+      from = SCM_CDR (SCM_CAR (scm_dynwinds));
+      wind_elt = SCM_CAR (scm_dynwinds);
+      scm_dynwinds = SCM_CDR (scm_dynwinds);
+#if 0
+      if (SCM_INUMP (wind_elt))
+	{
+	  scm_cross_dynwind_binding_scope (wind_elt, 0);
+	}
+      else
+#endif
+	{
+	  wind_key = SCM_CAR (wind_elt);
+	  if (   !(SCM_NIMP (wind_key) && SCM_SYMBOLP (wind_key))
+	      && (wind_key != SCM_BOOL_F)
+	      && (wind_key != SCM_BOOL_T))
+	    scm_apply (from, SCM_EOL, SCM_EOL);
+	}
       delta--;
       goto tail;		/* scm_dowinds(to, delta-1); */
     }
 }
 
 
-#ifdef __STDC__
-void
-scm_init_dynwind (void)
-#else
+
 void
 scm_init_dynwind ()
-#endif
 {
 #include "dynwind.x"
 }
