@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,122 +41,111 @@
 
 #include <stdio.h>
 #include "_scm.h"
+#include "ramap.h"
+#include "stackchk.h"
+#include "strorder.h"
+#include "smob.h"
+#include "unif.h"
+
+#include "eq.h"
 
-PROC1 (s_eq_p, "eq?", tc7_rpsubr, scm_eq_p);
-#ifdef __STDC__
-SCM
-scm_eq_p (SCM x, SCM y)
-#else
+SCM_PROC1 (s_eq_p, "eq?", scm_tc7_rpsubr, scm_eq_p);
+
 SCM
 scm_eq_p (x, y)
      SCM x;
      SCM y;
-#endif
 {
   return ((x==y)
-	  ? BOOL_T
-	  : BOOL_F);
+	  ? SCM_BOOL_T
+	  : SCM_BOOL_F);
 }
 
 
-PROC1 (s_eqv_p, "eqv?", tc7_rpsubr, scm_eqv_p);
-#ifdef __STDC__
-SCM
-scm_eqv_p (SCM x, SCM y)
-#else
+SCM_PROC1 (s_eqv_p, "eqv?", scm_tc7_rpsubr, scm_eqv_p);
+
 SCM
 scm_eqv_p (x, y)
      SCM x;
      SCM y;
-#endif
 {
-  if (x==y) return BOOL_T;
-  if IMP(x) return BOOL_F;
-  if IMP(y) return BOOL_F;
+  if (x==y) return SCM_BOOL_T;
+  if SCM_IMP(x) return SCM_BOOL_F;
+  if SCM_IMP(y) return SCM_BOOL_F;
   /* this ensures that types and scm_length are the same. */
-  if (CAR(x) != CAR(y)) return BOOL_F;
-  if NUMP(x) {
-# ifdef BIGDIG
-    if BIGP(x) return (0==scm_bigcomp(x, y)) ? BOOL_T : BOOL_F;
+  if (SCM_CAR(x) != SCM_CAR(y)) return SCM_BOOL_F;
+  if SCM_NUMP(x) {
+# ifdef SCM_BIGDIG
+    if SCM_BIGP(x) return (0==scm_bigcomp(x, y)) ? SCM_BOOL_T : SCM_BOOL_F;
 # endif
-#ifdef FLOATS
-    if (REALPART(x) != REALPART(y)) return BOOL_F;
-    if (CPLXP(x) && (IMAG(x) != IMAG(y))) return BOOL_F;
+#ifdef SCM_FLOATS
+    if (SCM_REALPART(x) != SCM_REALPART(y)) return SCM_BOOL_F;
+    if (SCM_CPLXP(x) && (SCM_IMAG(x) != SCM_IMAG(y))) return SCM_BOOL_F;
 #endif
-    return BOOL_T;
+    return SCM_BOOL_T;
   }
-  return BOOL_F;
+  return SCM_BOOL_F;
 }
 
-extern SCM scm_array_equal ();
 
-PROC1 (s_equal_p, "equal?", tc7_rpsubr, scm_equal_p);
-#ifdef __STDC__
-SCM
-scm_equal_p (SCM x, SCM y)
-#else
+SCM_PROC1 (s_equal_p, "equal?", scm_tc7_rpsubr, scm_equal_p);
+
 SCM
 scm_equal_p (x, y)
      SCM x;
      SCM y;
-#endif
 {
-  CHECK_STACK;
- tailrecurse: POLL;
-	if (x==y) return BOOL_T;
-	if (IMP(x)) return BOOL_F;
-	if (IMP(y)) return BOOL_F;
-	if (CONSP(x) && CONSP(y)) {
-		if FALSEP(scm_equal_p(CAR(x), CAR(y))) return BOOL_F;
-		x = CDR(x);
-		y = CDR(y);
+  SCM_CHECK_STACK;
+ tailrecurse: SCM_ASYNC_TICK;
+	if (x==y) return SCM_BOOL_T;
+	if (SCM_IMP(x)) return SCM_BOOL_F;
+	if (SCM_IMP(y)) return SCM_BOOL_F;
+	if (SCM_CONSP(x) && SCM_CONSP(y)) {
+		if SCM_FALSEP(scm_equal_p(SCM_CAR(x), SCM_CAR(y))) return SCM_BOOL_F;
+		x = SCM_CDR(x);
+		y = SCM_CDR(y);
 		goto tailrecurse;
 	}
 	/* this ensures that types and scm_length are the same. */
-	if (CAR(x) != CAR(y)) return BOOL_F;
-	switch (TYP7(x)) {
-        default: return BOOL_F;
-	case tc7_string: return scm_string_equal_p(x, y);
-	case tc7_vector:
-	case tc7_wvect:
+	if (SCM_CAR(x) != SCM_CAR(y)) return SCM_BOOL_F;
+	switch (SCM_TYP7(x)) {
+        default: return SCM_BOOL_F;
+	case scm_tc7_substring:
+	case scm_tc7_mb_substring:
+	case scm_tc7_mb_string:
+	case scm_tc7_string: return scm_string_equal_p(x, y);
+	case scm_tc7_vector:
+	case scm_tc7_wvect:
 	  return scm_vector_equal_p(x, y);
-	case tc7_lvector:
-	  {
-	    SCM hook;
-	    hook = scm_get_lvector_hook (x, LV_EQUAL_FN);
-	    if (hook == BOOL_F)
-	      return scm_vector_equal_p (x, y);
-	    else
-	      return scm_apply (hook,
-				scm_cons (x, scm_cons (y, EOL)),
-				EOL);
-	  }
-	case tc7_smob: {
-	        int i = SMOBNUM(x);
-	        if (!(i < scm_numsmob)) return BOOL_F;
+	case scm_tc7_smob: {
+	        int i = SCM_SMOBNUM(x);
+	        if (!(i < scm_numsmob)) return SCM_BOOL_F;
 	        if (scm_smobs[i].equalp)
 		  return (scm_smobs[i].equalp)(x, y);
 		else
-		  return BOOL_F;
+		  return SCM_BOOL_F;
 	      }
-	case tc7_bvect: case tc7_uvect: case tc7_ivect:
-	case tc7_fvect:	case tc7_cvect: case tc7_dvect:
-	case tc7_svect: case tc7_llvect: case tc7_byvect:
+	case scm_tc7_bvect: case scm_tc7_uvect: case scm_tc7_ivect:
+	case scm_tc7_fvect:	case scm_tc7_cvect: case scm_tc7_dvect:
+	case scm_tc7_svect:
+#ifdef LONGLONGS
+	case scm_tc7_llvect:
+#endif
+	case scm_tc7_byvect:
 	  if (   scm_tc16_array
 	      && scm_smobs[0x0ff & (scm_tc16_array >> 8)].equalp)
-	    return scm_array_equal(x, y);
+	    return scm_array_equal_p(x, y);
 	}
-	return BOOL_F;
+	return SCM_BOOL_F;
 }
 
 
-#ifdef __STDC__
-void
-scm_init_eq (void)
-#else
+
+
+
+
 void
 scm_init_eq ()
-#endif
 {
 #include "eq.x"
 }
