@@ -2,7 +2,7 @@
 
 #ifndef ERRORH
 #define ERRORH
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,56 +43,42 @@
  * If you do not wish that, delete this exception notice.  
  */
 
-#include "__scm.h"
-#include "pairs.h"
+#include "libguile/__scm.h"
 
 
-
-extern SCM *scm_loc_errobj;
-
-
-
-extern int scm_errjmp_bad;
 extern int scm_ints_disabled;
-extern int scm_sig_deferred;
-extern int scm_alrm_deferred;
-extern SCM scm_err_exp;
-extern SCM scm_err_env;
-extern char * scm_err_pos;
-extern char * scm_err_s_subr;
-extern SCM system_error_sym;
-extern SCM *scm_loc_errobj;
-extern scm_cell scm_tmp_errobj;
+
 
-#ifdef __STDC__
-extern void scm_han_sig (void);
-extern void scm_han_alrm (void);
-extern void scm_warn (char *str1, char *str2);
-extern SCM scm_errno (SCM arg);
-extern SCM scm_perror (SCM arg);
-extern void def_err_response (void);
-extern void scm_everr (SCM exp, SCM env, SCM arg, char *pos, char *s_subr);
-extern SCM scm_wta (SCM arg, char *pos, char *s_subr);
-extern void scm_init_error (void);
-
-#else /* STDC */
-extern void scm_han_sig ();
-extern void scm_han_alrm ();
-extern void scm_warn ();
-extern SCM scm_errno ();
-extern SCM scm_perror ();
-extern void def_err_response ();
-extern void scm_everr ();
-extern SCM scm_wta ();
-extern void scm_init_error ();
-
-#endif /* STDC */
-
-
-
-
 
+/* GCC can be told that a function doesn't return; this helps it do
+   better error checking (for uninitialized variable use, for
+   example), and some optimization.  */
+#ifdef __GNUC__
+#define SCM_NORETURN __attribute__ ((noreturn))
+#else
+#define SCM_NORETURN
+#endif
 
 
+extern SCM scm_errno SCM_P ((SCM arg));
+extern SCM scm_perror SCM_P ((SCM arg));
+extern void scm_error SCM_P ((SCM key, char *subr, char *message,
+			      SCM args, SCM rest)) SCM_NORETURN;
+extern void (*scm_error_callback) SCM_P ((SCM key, char *subr,
+					  char *message, SCM args, SCM rest));
+extern void scm_syserror SCM_P ((char *subr)) SCM_NORETURN;
+extern void scm_syserror_msg SCM_P ((char *subr, char *message, SCM args))
+                                   SCM_NORETURN;
+extern void scm_sysmissing SCM_P ((char *subr)) SCM_NORETURN;
+extern void scm_num_overflow SCM_P ((char *subr)) SCM_NORETURN;
+extern void scm_out_of_range SCM_P ((char *subr, SCM bad_value)) SCM_NORETURN;
+extern void scm_wrong_num_args SCM_P ((SCM proc)) SCM_NORETURN;
+extern void scm_wrong_type_arg SCM_P ((char *subr, int pos, SCM bad_value))
+                                     SCM_NORETURN;
+extern void scm_memory_error SCM_P ((char *subr)) SCM_NORETURN;
+extern void scm_misc_error SCM_P ((char *subr, char *message, SCM args))
+				 SCM_NORETURN;
+extern SCM scm_wta SCM_P ((SCM arg, char *pos, char *s_subr));
+extern void scm_init_error SCM_P ((void));
 
 #endif  /* ERRORH */
