@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,32 +43,57 @@
 #include <stdio.h>
 #include "_scm.h"
 
+#include "feature.h"
+
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
 
 
-SCM *scm_loc_features;
+static SCM *scm_loc_features;
 
-#ifdef __STDC__
-void
-scm_add_feature(char* str)
-#else
 void
 scm_add_feature(str)
      char* str;
-#endif
 {
-  *scm_loc_features = scm_cons(CAR(scm_intern(str, strlen(str))), *scm_loc_features);
+  *scm_loc_features = scm_cons(SCM_CAR(scm_intern(str, strlen(str))),
+			       *scm_loc_features);
 }
 
 
-#ifdef __STDC__
+
+SCM_PROC(s_program_arguments, "program-arguments", 0, 0, 0, scm_program_arguments);
+
+SCM 
+scm_program_arguments ()
+{
+  return scm_progargs;
+}
+
+/* Set the value returned by program-arguments, given ARGC and ARGV.
+
+   If FIRST is non-zero, make it the first element; we do this in
+   situations where other code (like getopt) has parsed out a few
+   arguments, but we still want the script name to be the first
+   element.  */
 void
-scm_init_feature(void)
-#else
+scm_set_program_arguments (argc, argv, first)
+     int argc;
+     char **argv;
+     char *first;
+{
+  scm_progargs = scm_makfromstrs (argc, argv);
+  if (first)
+    scm_progargs = scm_cons (scm_makfrom0str (first), scm_progargs);
+}
+
+
+
+
 void
 scm_init_feature()
-#endif
 {
-  scm_loc_features = &CDR(scm_sysintern("*features*", EOL));
+  scm_loc_features = SCM_CDRLOC (scm_sysintern ("*features*", SCM_EOL));
 #ifdef RECKLESS
   scm_add_feature("reckless");
 #endif
@@ -81,4 +106,16 @@ scm_init_feature()
 #ifdef SICP
   scm_add_feature("sicp");
 #endif
+#ifndef GO32
+  scm_add_feature("char-ready?");
+#endif
+#ifndef CHEAP_CONTINUATIONS
+  scm_add_feature ("full-continuation");
+#endif
+#ifdef USE_THREADS
+  scm_add_feature ("threads");
+#endif
+  
+  scm_sysintern ("char-code-limit", SCM_MAKINUM (SCM_CHAR_CODE_LIMIT));
+#include "feature.x"
 }
