@@ -2,7 +2,7 @@
 
 #ifndef FPORTSH
 #define FPORTSH
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,7 +44,9 @@
  */
 
 
-#include "__scm.h"
+#include "libguile/__scm.h"
+
+#include "libguile/ports.h"
 
 
 
@@ -52,27 +54,12 @@ extern scm_ptobfuns scm_fptob;
 extern scm_ptobfuns scm_pipob;
 
 
-#ifdef __STDC__
-extern SCM scm_setbuf0 (SCM port);
-extern long scm_mode_bits (char *modes);
-extern SCM scm_mkfile (char * name, char * modes);
-extern SCM scm_open_file (SCM filename, SCM modes);
-extern SCM scm_port_mode (SCM port);
-extern void scm_init_fports (void);
-
-#else /* STDC */
-extern SCM scm_setbuf0 ();
-extern long scm_mode_bits ();
-extern SCM scm_mkfile ();
-extern SCM scm_open_file ();
-extern SCM scm_port_mode ();
-extern void scm_init_fports ();
-
-#endif /* STDC */
-
-
-
-
 
+extern SCM scm_setbuf0 SCM_P ((SCM port));
+extern long scm_mode_bits SCM_P ((char *modes));
+extern SCM scm_open_file SCM_P ((SCM filename, SCM modes));
+extern SCM scm_stdio_to_port SCM_P ((FILE *file, char *name, char *modes));
+extern SCM scm_port_mode SCM_P ((SCM port));
+extern void scm_init_fports SCM_P ((void));
 
 #endif  /* FPORTSH */
