@@ -2,7 +2,7 @@
 
 #ifndef GSUBRH
 #define GSUBRH
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,22 +44,12 @@
  */
 
 
-#include "__scm.h"
+#include "libguile/__scm.h"
 
 
-#ifdef __STDC__
-extern SCM scm_make_gsubr(char *name, int req, int opt, int rst, SCM (*fcn)());
-extern SCM scm_gsubr_apply(SCM args);
-extern void scm_init_gsubr(void);
-
-#else /* STDC */
-extern SCM scm_make_gsubr();
-extern SCM scm_gsubr_apply();
-extern void scm_init_gsubr();
-
-#endif /* STDC */
-
-
 
+extern SCM scm_make_gsubr SCM_P ((char *name, int req, int opt, int rst, SCM (*fcn)()));
+extern SCM scm_gsubr_apply SCM_P ((SCM args));
+extern void scm_init_gsubr SCM_P ((void));
 
 #endif  /* GSUBRH */
