@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,98 +43,143 @@
 #include <stdio.h>
 #include "_scm.h"
 
-
+#include "procs.h"
 
 
 
 /* {Procedures}
  */
 
-#ifdef __STDC__
+
 SCM 
-scm_make_subr (char *name, int type, SCM (*fcn) ())
-#else
+scm_make_subr_opt (name, type, fcn, set)
+     char *name;
+     int type;
+     SCM (*fcn) ();
+     int set;
+{
+  SCM symcell;
+  long tmp;
+  register SCM z;
+  symcell = scm_sysintern (name, SCM_UNDEFINED);
+  tmp = ((((SCM_CELLPTR) (SCM_CAR (symcell))) - scm_heap_org) << 8);
+  if ((tmp >> 8) != ((SCM_CELLPTR) (SCM_CAR (symcell)) - scm_heap_org))
+    tmp = 0;
+  SCM_NEWCELL (z);
+  SCM_SUBRF (z) = fcn;
+  SCM_SETCAR (z, tmp + type);
+  if (set)
+    SCM_SETCDR (symcell, z);
+  return z;
+}
+
+
+
 SCM 
 scm_make_subr (name, type, fcn)
      char *name;
      int type;
      SCM (*fcn) ();
-#endif
 {
-  SCM symcell = scm_sysintern (name, SCM_UNDEFINED);
-  long tmp = ((((CELLPTR) (CAR (symcell))) - scm_heap_org) << 8);
-  register SCM z;
-  if ((tmp >> 8) != ((CELLPTR) (CAR (symcell)) - scm_heap_org))
-    tmp = 0;
-  NEWCELL (z);
-  SUBRF (z) = fcn;
-  CAR (z) = tmp + type;
-  CDR (symcell) = z;
-  return z;
+  return scm_make_subr_opt (name, type, fcn, 1);
 }
 
 #ifdef CCLO
-#ifdef __STDC__
-SCM 
-scm_makcclo (SCM proc, long len)
-#else
+
 SCM 
 scm_makcclo (proc, len)
      SCM proc;
      long len;
-#endif
 {
   SCM s;
-  NEWCELL (s);
-  DEFER_INTS;
-  SETCHARS (s, scm_must_malloc (len * sizeof (SCM), "compiled-closure"));
-  SETLENGTH (s, len, tc7_cclo);
+  SCM_NEWCELL (s);
+  SCM_DEFER_INTS;
+  SCM_SETCHARS (s, scm_must_malloc (len * sizeof (SCM), "compiled-closure"));
+  SCM_SETLENGTH (s, len, scm_tc7_cclo);
   while (--len)
-    VELTS (s)[len] = UNSPECIFIED;
-  CCLO_SUBR (s) = proc;
-  ALLOW_INTS;
+    SCM_VELTS (s)[len] = SCM_UNSPECIFIED;
+  SCM_CCLO_SUBR (s) = proc;
+  SCM_ALLOW_INTS;
   return s;
 }
 #endif
 
 
 
-PROC (s_procedure_p, "procedure?", 1, 0, 0, scm_procedure_p);
-#ifdef __STDC__
-SCM 
-scm_procedure_p (SCM obj)
-#else
+SCM_PROC(s_procedure_p, "procedure?", 1, 0, 0, scm_procedure_p);
+
 SCM 
 scm_procedure_p (obj)
      SCM obj;
+{
+  if (SCM_NIMP (obj))
+    switch (SCM_TYP7 (obj))
+      {
+      case scm_tcs_closures:
+      case scm_tc7_contin:
+      case scm_tcs_subrs:
+#ifdef CCLO
+      case scm_tc7_cclo:
+#endif
+	return SCM_BOOL_T;
+      default:
+	return SCM_BOOL_F;
+      }
+  return SCM_BOOL_F;
+}
+
+SCM_PROC(s_closure_p, "closure?", 1, 0, 0, scm_closure_p);
+
+SCM 
+scm_closure_p (obj)
+     SCM obj;
+{
+  if (SCM_NIMP (obj))
+    switch (SCM_TYP7 (obj))
+      {
+      case scm_tcs_closures:
+	return SCM_BOOL_T;
+      default: ;
+      }
+  return SCM_BOOL_F;
+}
+
+#ifdef __STDC__
+SCM 
+scm_thunk_p (SCM obj)
+#else
+SCM 
+scm_thunk_p (obj)
+     SCM obj;
 #endif
 {
-  if (NIMP (obj))
-    switch (TYP7 (obj))
+  if (SCM_NIMP (obj))
+    switch (SCM_TYP7 (obj))
       {
-      case tcs_closures:
-      case tc7_contin:
-      case tcs_subrs:
+      case scm_tcs_closures:
+	if (SCM_NULLP (SCM_CAR (SCM_CODE (obj))))
+	  return SCM_BOOL_T;
+      case scm_tc7_subr_0:
+      case scm_tc7_subr_1o:
+      case scm_tc7_lsubr:
+      case scm_tc7_rpsubr:
+      case scm_tc7_asubr:
 #ifdef CCLO
-      case tc7_cclo:
+      case scm_tc7_cclo:
 #endif
-	return BOOL_T;
+	return SCM_BOOL_T;
       default:
-	return BOOL_F;
+	;
       }
-  return BOOL_F;
+  return SCM_BOOL_F;
 }
 
 
-#ifdef __STDC__
-void
-scm_init_iprocs(scm_iproc *subra, int type)
-#else
+
 void
 scm_init_iprocs(subra, type)
      scm_iproc *subra;
      int type;
-#endif
 {
   for(;subra->scm_string; subra++)
     scm_make_subr(subra->scm_string,
@@ -145,13 +190,9 @@ scm_init_iprocs(subra, type)
 
 
 
-#ifdef __STDC__
-void
-scm_init_procs (void)
-#else
+
 void
 scm_init_procs ()
-#endif
 {
 #include "procs.x"
 }
