@@ -2,7 +2,7 @@
 
 #ifndef PROCSH
 #define PROCSH
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@
  */
 
 
-#include "__scm.h"
+#include "libguile/__scm.h"
 
 
 
@@ -70,7 +70,7 @@ typedef struct scm_dsubr
   double (*dproc) ();
 } scm_dsubr;
 
-#define SCM_SNAME(x) ((SCM_CAR(x)>>8)?(SCM)(scm_heap_org+(SCM_CAR(x)>>8)):nullstr)
+#define SCM_SNAME(x) ((SCM_CAR(x)>>8)?(SCM)(scm_heap_org+(SCM_CAR(x)>>8)):scm_nullstr)
 #define SCM_SUBRF(x) (((scm_subr *)(SCM2PTR(x)))->cproc)
 #define SCM_DSUBRF(x) (((scm_dsubr *)(SCM2PTR(x)))->dproc)
 #define SCM_CCLO_SUBR(x) (SCM_VELTS(x)[0])
@@ -82,31 +82,22 @@ typedef struct scm_dsubr
 #define SCM_CLOSCAR(x) (SCM_CAR(x)-scm_tc3_closure)
 #define SCM_CODE(x) SCM_CAR(SCM_CLOSCAR (x))
 #define SCM_PROCPROPS(x) SCM_CDR(SCM_CLOSCAR (x))
-#define SCM_SETCODE(x, e) SCM_CAR(x) = (scm_cons ((e), SCM_EOL) + scm_tc3_closure)
+#define SCM_SETPROCPROPS(x, p) SCM_SETCDR(SCM_CLOSCAR (x), p)
+#define SCM_SETCODE(x, e) (SCM_SETCAR (x, scm_cons ((e), SCM_EOL) + scm_tc3_closure))
 #define SCM_ENV(x) SCM_CDR(x)
-#define SCM_TOP_LEVEL(ENV)  (SCM_NULLP(ENV) || (SCM_BOOL_T == scm_procedurep (SCM_CAR (ENV))))
+#define SCM_SETENV(x, e) SCM_SETCDR (x, e)
+#define SCM_TOP_LEVEL(SCM_ENV)  (SCM_NULLP(SCM_ENV) || (SCM_BOOL_T == scm_procedure_p (SCM_CAR (SCM_ENV))))
 
 
 
+extern SCM scm_make_subr SCM_P ((char *name, int type, SCM (*fcn) ()));
+extern SCM scm_make_subr_opt SCM_P ((char *name, int type, SCM (*fcn) (),
+				     int set));
+extern SCM scm_makcclo SCM_P ((SCM proc, long len));
+extern SCM scm_procedure_p SCM_P ((SCM obj));
+extern SCM scm_thunk_p SCM_P ((SCM obj));
+extern void scm_init_iprocs SCM_P ((scm_iproc *subra, int type));
+extern void scm_init_procs SCM_P ((void));
+
 
-#ifdef __STDC__
-extern SCM scm_make_subr (char *name, int type, SCM (*fcn) ());
-extern SCM scm_makcclo (SCM proc, long len);
-extern SCM scm_procedure_p (SCM obj);
-extern void scm_init_iprocs(scm_iproc *subra, int type);
-extern void scm_init_procs (void);
-
-#else /* STDC */
-extern SCM scm_make_subr ();
-extern SCM scm_makcclo ();
-extern SCM scm_procedure_p ();
-extern void scm_init_iprocs();
-extern void scm_init_procs ();
-
-#endif /* STDC */
-
-
-
-
-
 #endif  /* PROCSH */
