@@ -2,7 +2,7 @@
 
 #ifndef RAMAPH
 #define RAMAPH
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,14 +44,30 @@
  */
 
 
-#include "__scm.h"
+#include "libguile/__scm.h"
 
 
-#ifdef __STDC__
-SCM scm_raequal (SCM ra0, SCM ra1);
-#else /* STDC */
-SCM scm_raequal ();
-#endif /* STDC */
 
+extern int scm_ra_matchp SCM_P ((SCM ra0, SCM ras));
+extern int scm_ramapc SCM_P ((int (*cproc) (), SCM data, SCM ra0, SCM lra, char *what));
+extern int scm_array_fill_int SCM_P ((SCM ra, SCM fill, SCM ignore));
+extern SCM scm_array_fill_x SCM_P ((SCM ra, SCM fill));
+extern SCM scm_array_copy_x SCM_P ((SCM src, SCM dst));
+extern int scm_ra_eqp SCM_P ((SCM ra0, SCM ras));
+extern int scm_ra_lessp SCM_P ((SCM ra0, SCM ras));
+extern int scm_ra_leqp SCM_P ((SCM ra0, SCM ras));
+extern int scm_ra_grp SCM_P ((SCM ra0, SCM ras));
+extern int scm_ra_greqp SCM_P ((SCM ra0, SCM ras));
+extern int scm_ra_sum SCM_P ((SCM ra0, SCM ras));
+extern int scm_ra_difference SCM_P ((SCM ra0, SCM ras));
+extern int scm_ra_product SCM_P ((SCM ra0, SCM ras));
+extern int scm_ra_divide SCM_P ((SCM ra0, SCM ras));
+extern int scm_array_identity SCM_P ((SCM src, SCM dst));
+extern SCM scm_array_map SCM_P ((SCM ra0, SCM proc, SCM lra));
+extern SCM scm_array_for_each SCM_P ((SCM proc, SCM ra0, SCM lra));
+extern SCM scm_array_index_map_x SCM_P ((SCM ra, SCM proc));
+extern SCM scm_raequal SCM_P ((SCM ra0, SCM ra1));
+extern SCM scm_array_equal_p SCM_P ((SCM ra0, SCM ra1));
+extern void scm_init_ramap SCM_P ((void));
 
 #endif  /* RAMAPH */
