@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,6 +44,8 @@
 #include <signal.h>
 #include "_scm.h"
 
+#include "scmsigs.h"
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -55,6 +57,11 @@
 #define signal ssignal		/* Needed for TURBOC V1.0 */
 #endif
 
+#ifdef USE_MIT_PTHREADS
+#undef signal
+#define signal pthread_signal
+#endif
+
 
 
 /* SIGRETTYPE is the type that signal handlers return.  See <signal.h>*/
@@ -83,282 +90,160 @@
 #endif
 #endif
 
+
+
+#define SIGFN(NAME, SCM_NAME, SIGNAL) \
+static SIGRETTYPE \
+NAME (sig) \
+     int sig; \
+{ \
+  signal (SIGNAL, NAME); \
+  scm_take_signal (SCM_NAME); \
+}
+
 #ifdef SIGHUP
-#ifdef __STDC__
-static SIGRETTYPE 
-hup_signal (int sig)
-#else
-static SIGRETTYPE 
-hup_signal (sig)
-     int sig;
-#endif
-{
-  signal (SIGHUP, hup_signal);
-  scm_wta (SCM_UNDEFINED, (char *) HUP_SIGNAL, "");
-}
+SIGFN(scm_hup_signal, SCM_HUP_SIGNAL, SIGHUP)
 #endif
 
-
-#ifdef __STDC__
-static SIGRETTYPE 
-int_signal (int sig)
-#else
-static SIGRETTYPE 
-int_signal (sig)
-     int sig;
-#endif
-{
-  sig = errno;
-  signal (SIGINT, int_signal);
-  if (scm_ints_disabled)
-    scm_sig_deferred = 1;
-  else
-    scm_han_sig ();
-  errno = sig;
-}
-
-/* If doesn't have SIGFPE, disable FLOATS for the rest of this file. */
-
-#ifndef SIGFPE
-#undef FLOATS
+#ifdef SIGINT
+SIGFN(scm_int_signal, SCM_INT_SIGNAL, SIGINT)
 #endif
 
-#ifdef FLOATS
-#ifdef __STDC__
-static SIGRETTYPE 
-fpe_signal (int sig)
-#else
-static SIGRETTYPE 
-fpe_signal (sig)
-     int sig;
+#ifdef SIGFPE
+SIGFN(scm_fpe_signal, SCM_FPE_SIGNAL, SIGFPE)
 #endif
-{
-  signal (SIGFPE, fpe_signal);
-  scm_wta (SCM_UNDEFINED, (char *) FPE_SIGNAL, "");
-}
-#endif
-
 
 #ifdef SIGBUS
-#ifdef __STDC__
-static SIGRETTYPE 
-bus_signal (int sig)
-#else
-static SIGRETTYPE 
-bus_signal (sig)
-     int sig;
-#endif
-{
-  signal (SIGBUS, bus_signal);
-  scm_wta (SCM_UNDEFINED, (char *) BUS_SIGNAL, "");
-}
+SIGFN(scm_bus_signal, SCM_BUS_SIGNAL, SIGBUS)
 #endif
 
+#ifdef SIGSEGV
+SIGFN(scm_segv_signal, SCM_SEGV_SIGNAL, SIGSEGV)
+#endif
 
-#ifdef SIGSEGV			/* AMIGA lacks! */
-#ifdef __STDC__
-static SIGRETTYPE 
-segv_signal (int sig)
-#else
-static SIGRETTYPE 
-segv_signal (sig)
-     int sig;
-#endif
-{
-  signal (SIGSEGV, segv_signal);
-  scm_wta (SCM_UNDEFINED, (char *) SEGV_SIGNAL, "");
-}
-#endif
-#ifdef atarist
-#undef SIGALRM			/* only available via MiNT libs */
-#endif
-#ifdef GO32
-#undef SIGALRM
-#endif
-#ifdef __HIGHC__
-# undef SIGALRM
-#endif
 #ifdef SIGALRM
-#ifdef __STDC__
-static SIGRETTYPE 
-alrm_signal (int sig)
-#else
-static SIGRETTYPE 
-alrm_signal (sig)
-     int sig;
+SIGFN(scm_alrm_signal, SCM_ALRM_SIGNAL, SIGALRM)
 #endif
-{
-  sig = errno;
-  signal (SIGALRM, alrm_signal);
-  if (scm_ints_disabled)
-    scm_alrm_deferred = 1;
-  else
-    scm_han_alrm ();
-  errno = sig;
+
+#define FAKESIGFN(NAME, SCM_NAME) \
+static SIGRETTYPE \
+NAME (sig) \
+     int sig; \
+{ \
+  scm_take_signal (SCM_NAME); \
 }
 
-PROC (s_alarm, "alarm", 1, 0, 0, scm_alarm);
-#ifdef __STDC__
-SCM 
-scm_alarm (SCM i)
-#else
+#if 0
+/* !!! */
+FAKESIGFN(scm_gc_signal, SCM_GC_SIGNAL)
+FAKESIGFN(scm_tick_signal, SCM_TICK_SIGNAL)
+#endif
+
+
+SCM_PROC(s_alarm, "alarm", 1, 0, 0, scm_alarm);
+
 SCM 
 scm_alarm (i)
      SCM i;
-#endif
 {
   unsigned int j;
-  ASSERT (INUMP (i) && (INUM (i) >= 0), i, ARG1, s_alarm);
-  SYSCALL (j = alarm (INUM (i)));
-  return MAKINUM (j);
+  SCM_ASSERT (SCM_INUMP (i) && (SCM_INUM (i) >= 0), i, SCM_ARG1, s_alarm);
+  SCM_SYSCALL (j = alarm (SCM_INUM (i)));
+  return SCM_MAKINUM (j);
 }
 
 
-#ifndef AMIGA
-PROC (s_pause, "pause", 0, 0, 0, scm_pause);
-#ifdef __STDC__
-SCM 
-scm_pause (void)
-#else
+SCM_PROC(s_pause, "pause", 0, 0, 0, scm_pause);
+
 SCM 
 scm_pause ()
-#endif
 {
   pause ();
-  return UNSPECIFIED;
+  return SCM_UNSPECIFIED;
 }
-#endif
-#endif /* SIGALRM */
 
-#ifndef AMIGA
-# ifndef _Windows
+SCM_PROC(s_sleep, "sleep", 1, 0, 0, scm_sleep);
 
-PROC (s_sleep, "sleep", 1, 0, 0, scm_sleep);
-#ifdef __STDC__
-SCM 
-scm_sleep (SCM i)
-#else
 SCM 
 scm_sleep (i)
      SCM i;
-#endif
 {
   unsigned int j;
-  ASSERT (INUMP (i) && (INUM (i) >= 0), i, ARG1, s_sleep);
+  SCM_ASSERT (SCM_INUMP (i) && (SCM_INUM (i) >= 0), i, SCM_ARG1, s_sleep);
 #ifdef __HIGHC__
-  SYSCALL(j = 0; sleep(INUM(i)););
+  SCM_SYSCALL(j = 0; sleep(SCM_INUM(i)););
 #else
-  SYSCALL(j = sleep(INUM(i)););
+  SCM_SYSCALL(j = sleep(SCM_INUM(i)););
 #endif
-  return MAKINUM (j);
+  return SCM_MAKINUM (j);
 }
-# endif
-#endif
 
+SCM_PROC(s_raise, "raise", 1, 0, 0, scm_raise);
 
-
-#ifndef GO32
-/* int raise P((int sig)); */
-PROC (s_raise, "raise", 1, 0, 0, scm_raise);
-#ifdef __STDC__
-SCM
-scm_raise(SCM sig)
-#else
 SCM
 scm_raise(sig)
      SCM sig;
-#endif
 {
-  ASSERT(INUMP(sig), sig, ARG1, s_raise);
+  SCM_ASSERT(SCM_INUMP(sig), sig, SCM_ARG1, s_raise);
 # ifdef vms
-  return MAKINUM(gsignal((int)INUM(sig)));
+  return SCM_MAKINUM(gsignal((int)SCM_INUM(sig)));
 # else
-  return kill (getpid(), (int)INUM(sig)) ? BOOL_F : BOOL_T;
+  return kill (getpid(), (int)SCM_INUM(sig)) ? SCM_BOOL_F : SCM_BOOL_T;
 # endif
 }
-#endif
-
-#ifdef TICKS
-unsigned int scm_tick_count = 0, scm_ticken = 0;
-SCM *scm_loc_tick_signal;
-#ifdef __STDC__
-void 
-scm_tick_signal (void)
-#else
-void 
-scm_tick_signal ()
-#endif
-{
-  if (scm_ticken && NIMP (*scm_loc_tick_signal))
-    {
-      scm_ticken = 0;
-      scm_apply (*scm_loc_tick_signal, EOL, EOL);
-    }
-}
-
-
-PROC (s_ticks, "ticks", 1, 0, 0, scm_ticks);
-#ifdef __STDC__
-SCM 
-scm_ticks (SCM i)
-#else
-SCM 
-scm_ticks (i)
-     SCM i;
-#endif
-{
-  SCM j = scm_ticken ? scm_tick_count : 0;
-  if (!UNBNDP (i))
-    scm_ticken = scm_tick_count = INUM (i);
-  return MAKINUM (j);
-}
-#endif
 
+
 #ifdef SIGHUP
 static SIGRETTYPE (*oldhup) ();
 #endif
+
+#ifdef SIGINT
 static SIGRETTYPE (*oldint) ();
-#ifdef FLOATS
+#endif
+
+#ifdef SIGFPE
 static SIGRETTYPE (*oldfpe) ();
 #endif
+
 #ifdef SIGBUS
 static SIGRETTYPE (*oldbus) ();
 #endif
+
 #ifdef SIGSEGV			/* AMIGA lacks! */
 static SIGRETTYPE (*oldsegv) ();
 #endif
+
 #ifdef SIGALRM
 static SIGRETTYPE (*oldalrm) ();
 #endif
+
 #ifdef SIGPIPE
 static SIGRETTYPE (*oldpipe) ();
 #endif
 
 
-#ifdef __STDC__
-void 
-scm_init_signals (void)
-#else
+
 void 
 scm_init_signals ()
-#endif
 {
-  oldint = signal (SIGINT, int_signal);
-#ifdef SIGHUP
-  oldhup = signal (SIGHUP, hup_signal);
+#ifdef SIGINT
+  oldint = signal (SIGINT, scm_int_signal);
 #endif
-#ifdef FLOATS
-  oldfpe = signal (SIGFPE, fpe_signal);
+#ifdef SIGHUP
+  oldhup = signal (SIGHUP, scm_hup_signal);
+#endif
+#ifdef SIGFPE
+  oldfpe = signal (SIGFPE, scm_fpe_signal);
 #endif
 #ifdef SIGBUS
-  oldbus = signal (SIGBUS, bus_signal);
+  oldbus = signal (SIGBUS, scm_bus_signal);
 #endif
 #ifdef SIGSEGV			/* AMIGA lacks! */
-  oldsegv = signal (SIGSEGV, segv_signal);
+  oldsegv = signal (SIGSEGV, scm_segv_signal);
 #endif
 #ifdef SIGALRM
   alarm (0);			/* kill any pending ALRM interrupts */
-  oldalrm = signal (SIGALRM, alrm_signal);
+  oldalrm = signal (SIGALRM, scm_alrm_signal);
 #endif
 #ifdef SIGPIPE
   oldpipe = signal (SIGPIPE, SIG_IGN);
@@ -374,13 +259,9 @@ scm_init_signals ()
 /* This is used in preparation for a possible fork().  Ignore all
    signals before the fork so that child will catch only if it
    establishes a handler */
-#ifdef __STDC__
-void 
-scm_ignore_signals (void)
-#else
+
 void 
 scm_ignore_signals ()
-#endif
 {
 #ifdef ultrix
   siginterrupt (SIGINT, 0);
@@ -392,7 +273,7 @@ scm_ignore_signals ()
 #ifdef SIGHUP
   signal (SIGHUP, SIG_DFL);
 #endif
-#ifdef FLOATS
+#ifdef SCM_FLOATS
   signal (SIGFPE, SIG_DFL);
 #endif
 #ifdef SIGBUS
@@ -415,29 +296,25 @@ scm_ignore_signals ()
   fflush (stderr);
 }
 
-#ifdef __STDC__
-void 
-scm_unignore_signals (void)
-#else
+
 void 
 scm_unignore_signals ()
-#endif
 {
-  signal (SIGINT, int_signal);
+  signal (SIGINT, scm_int_signal);
 #ifdef SIGHUP
-  signal (SIGHUP, hup_signal);
+  signal (SIGHUP, scm_hup_signal);
 #endif
-#ifdef FLOATS
-  signal (SIGFPE, fpe_signal);
+#ifdef SCM_FLOATS
+  signal (SIGFPE, scm_fpe_signal);
 #endif
 #ifdef SIGBUS
-  signal (SIGBUS, bus_signal);
+  signal (SIGBUS, scm_bus_signal);
 #endif
 #ifdef SIGSEGV			/* AMIGA lacks! */
-  signal (SIGSEGV, segv_signal);
+  signal (SIGSEGV, scm_segv_signal);
 #endif
 #ifdef SIGALRM
-  signal (SIGALRM, alrm_signal);
+  signal (SIGALRM, scm_alrm_signal);
 #endif
 #ifdef ultrix
   siginterrupt (SIGINT, 1);
@@ -447,13 +324,10 @@ scm_unignore_signals ()
 #endif /* ultrix */
 }
 
-#ifdef __STDC__
-void 
-scm_restore_signals (void)
-#else
-void 
+SCM_PROC (s_restore_signals, "restore-signals", 0, 0, 0, scm_restore_signals);
+
+SCM
 scm_restore_signals ()
-#endif
 {
 #ifdef ultrix
   siginterrupt (SIGINT, 0);
@@ -465,7 +339,7 @@ scm_restore_signals ()
 #ifdef SIGHUP
   signal (SIGHUP, oldhup);
 #endif
-#ifdef FLOATS
+#ifdef SCM_FLOATS
   signal (SIGFPE, oldfpe);
 #endif
 #ifdef SIGBUS
@@ -481,21 +355,14 @@ scm_restore_signals ()
   alarm (0);			/* kill any pending ALRM interrupts */
   signal (SIGALRM, oldalrm);
 #endif
+  return SCM_UNSPECIFIED;
 }
 
 
-#ifdef __STDC__
-void
-scm_init_scmsigs (void)
-#else
+
 void
 scm_init_scmsigs ()
-#endif
 {
-#ifdef TICKS
-  scm_loc_tick_signal = &CDR(scm_sysintern("ticks-interrupt", SCM_UNDEFINED));
-  scm_make_subr(s_ticks, tc7_subr_1o, scm_ticks);
-#endif
 #include "scmsigs.x"
 }
 
