@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,6 +42,9 @@
 
 #include <stdio.h>
 #include "_scm.h"
+
+#include "smob.h"
+
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
 #endif
@@ -53,23 +56,19 @@
  * Indexes into this table are used when generating type
  * tags for smobjects (if you know a tag you can get an index and conversely).
  */
-sizet scm_numsmob;
+scm_sizet scm_numsmob;
 scm_smobfuns *scm_smobs;
 
-#ifdef __STDC__
-long 
-scm_newsmob (scm_smobfuns *smob)
-#else
+
 long 
 scm_newsmob (smob)
      scm_smobfuns *smob;
-#endif
 {
   char *tmp;
   if (255 <= scm_numsmob)
     goto smoberr;
-  DEFER_INTS;
-  SYSCALL (tmp = (char *) realloc ((char *) scm_smobs, (1 + scm_numsmob) * sizeof (scm_smobfuns)));
+  SCM_DEFER_INTS;
+  SCM_SYSCALL (tmp = (char *) realloc ((char *) scm_smobs, (1 + scm_numsmob) * sizeof (scm_smobfuns)));
   if (tmp)
     {
       scm_smobs = (scm_smobfuns *) tmp;
@@ -79,10 +78,10 @@ scm_newsmob (smob)
       scm_smobs[scm_numsmob].equalp = smob->equalp;
       scm_numsmob++;
     }
-  ALLOW_INTS;
+  SCM_ALLOW_INTS;
   if (!tmp)
-  smoberr:scm_wta (MAKINUM ((long) scm_numsmob), (char *) NALLOC, "newsmob");
-  return tc7_smob + (scm_numsmob - 1) * 256;
+  smoberr:scm_wta (SCM_MAKINUM ((long) scm_numsmob), (char *) SCM_NALLOC, "newsmob");
+  return scm_tc7_smob + (scm_numsmob - 1) * 256;
 }
 
 /* {Initialization for i/o types, float, bignum, the type of free cells}
@@ -114,13 +113,9 @@ static scm_smobfuns bigob =
 
 
 
-#ifdef __STDC__
-void
-scm_smob_prehistory (void)
-#else
+
 void
 scm_smob_prehistory ()
-#endif
 {
   scm_numsmob = 0;
   scm_smobs = (scm_smobfuns *) malloc (7 * sizeof (scm_smobfuns));
