@@ -2,7 +2,7 @@
 
 #ifndef SMOBH
 #define SMOBH
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,55 +43,36 @@
  * If you do not wish that, delete this exception notice.  
  */
 
-#include "__scm.h"
-#include "params.h"
+#include "libguile/__scm.h"
 
 
-
-
-
 typedef struct scm_smobfuns
 {
-  SCM (*mark) P ((SCM));
-  sizet (*free) P ((SCM));
-  int (*print) P ((SCM exp, SCM port, int writing));
-  SCM (*equalp) P ((SCM, SCM));
+  SCM (*mark) SCM_P ((SCM));
+  scm_sizet (*free) SCM_P ((SCM));
+  int (*print) SCM_P ((SCM exp, SCM port, scm_print_state *pstate));
+  SCM (*equalp) SCM_P ((SCM, SCM));
 } scm_smobfuns;
 
-typedef struct scm_ptobfuns
-{
-  SCM (*mark) ();
-  int (*free) ();
-  int (*print) ();
-  SCM (*equalp) ();
-  int (*fputc) ();
-  int (*fputs) ();
-  sizet (*fwrite) ();
-  int (*fflush) ();
-  int (*fgetc) ();
-  int (*fclose) ();
-} scm_ptobfuns;
-
-
 
 
-extern sizet scm_numsmob;
+#define SCM_SMOBNUM(x) (0x0ff & (SCM_CAR(x)>>8));
+#define SCM_PTOBNUM(x) (0x0ff & (SCM_CAR(x)>>8));
+
+extern scm_sizet scm_numsmob;
 extern scm_smobfuns *scm_smobs;
 
 
-#ifdef __STDC__
-extern long scm_newsmob (scm_smobfuns *smob);
-extern void scm_smob_prehistory (void);
-
-#else /* STDC */
-extern long scm_newsmob ();
-extern void scm_smob_prehistory ();
-
-#endif /* STDC */
-
-
 
+/* Everyone who uses smobs needs to print.  */
+#include "libguile/ports.h"
+#include "libguile/genio.h"
 
+/* ... and they all need to GC.  */
+#include "libguile/markers.h"
 
+
+extern long scm_newsmob SCM_P ((scm_smobfuns *smob));
+extern void scm_smob_prehistory SCM_P ((void));
 
 #endif  /* SMOBH */
