@@ -2,7 +2,7 @@
 
 #ifndef STACKCHKH
 #define STACKCHKH
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,27 +43,50 @@
  * If you do not wish that, delete this exception notice.  
  */
 
-#include "__scm.h"
+#include "libguile/__scm.h"
+
+#include "libguile/continuations.h"
+#ifdef DEBUG_EXTENSIONS
+#include "libguile/debug.h"
+#endif
+
+
+/* With debug extensions we have the possibility to use the debug options
+ * to disable stack checking.
+ */
+#ifdef DEBUG_EXTENSIONS
+#define SCM_STACK_CHECKING_P SCM_STACK_LIMIT
+#else
+/* *fixme* This option should be settable also without debug extensions. */
+#define SCM_STACK_LIMIT 100000
+#define SCM_STACK_CHECKING_P 1
+#endif
+
+#ifdef STACK_CHECKING
+# ifdef SCM_STACK_GROWS_UP
+#  define SCM_STACK_OVERFLOW_P(s)\
+   (s - SCM_BASE (scm_rootcont) > SCM_STACK_LIMIT * sizeof (SCM_STACKITEM))
+# else
+#  define SCM_STACK_OVERFLOW_P(s)\
+   (SCM_BASE (scm_rootcont) - s > SCM_STACK_LIMIT * sizeof (SCM_STACKITEM))
+# endif
+# define SCM_CHECK_STACK\
+    {\
+       SCM_STACKITEM stack;\
+       if (SCM_STACK_OVERFLOW_P (&stack) && scm_stack_checking_enabled_p)\
+	 scm_report_stack_overflow ();\
+    }
+#else
+# define SCM_CHECK_STACK /**/
+#endif /* STACK_CHECKING */
+
+extern int scm_stack_checking_enabled_p;
 
 
-#ifdef __STDC__
-extern void scm_stack_check (void);
-extern long scm_stack_size (SCM_STACKITEM *start);
-extern void scm_stack_report (void);
-extern void scm_init_stackchk (void);
-
-#else /* STDC */
-extern void scm_stack_check ();
-extern long scm_stack_size ();
-extern void scm_stack_report ();
-extern void scm_init_stackchk ();
-
-#endif /* STDC */
-
-
-
-
-
 
+extern void scm_report_stack_overflow SCM_P ((void));
+extern long scm_stack_size SCM_P ((SCM_STACKITEM *start));
+extern void scm_stack_report SCM_P ((void));
+extern void scm_init_stackchk SCM_P ((void));
 
 #endif  /* STACKCHKH */
