@@ -2,7 +2,7 @@
 
 #ifndef STRINGSH
 #define STRINGSH
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,80 +44,36 @@
  */
 
 
-#include "__scm.h"
+#include "libguile/__scm.h"
+
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
 
 
 
-#define SCM_STRINGP(x) (SCM_TYP7(x)==scm_tc7_string)
+#define SCM_STRINGP(x) (SCM_TYP7S(x)==scm_tc7_string)
 #define SCM_NSTRINGP(x) (!SCM_STRINGP(x))
 
 
 
-#ifdef __STDC__
-extern SCM scm_makstr (long len, int slots);
-extern SCM scm_makfromstrs (int argc, char **argv);
-extern SCM scm_take0str (char * it);
-extern SCM scm_makfromstr (char *src, sizet len, int slots);
-extern SCM makfrom0str (char *src);
-extern SCM makfrom0str_opt (char *src);
-extern SCM scm_string_p (SCM x);
-extern SCM scm_string (SCM chrs);
-extern SCM scm_make_string (SCM k, SCM chr);
-extern SCM scm_string_length (SCM str);
-extern SCM scm_string_ref (SCM str, SCM k);
-extern SCM scm_string_set (SCM str, SCM k, SCM chr);
-extern SCM scm_string_equal_p (SCM s1, SCM s2);
-extern SCM scm_string_ci_equal_p (SCM s1, SCM s2);
-extern SCM scm_string_less_p (SCM s1, SCM s2);
-extern SCM scm_string_leq_p (SCM s1, SCM s2);
-extern SCM scm_string_gr_p (SCM s1, SCM s2);
-extern SCM scm_string_geq_p (SCM s1, SCM s2);
-extern SCM scm_string_ci_less_p (SCM s1, SCM s2);
-extern SCM scm_string_ci_leq_p (SCM s1, SCM s2);
-extern SCM scm_string_ci_gr_p (SCM s1, SCM s2);
-extern SCM scm_string_ci_geq_p (SCM s1, SCM s2);
-extern SCM scm_substring (SCM str, SCM start, SCM end);
-extern SCM scm_string_append (SCM args);
-extern void scm_init_strings (void);
-
-#else /* STDC */
-extern SCM scm_makstr ();
-extern SCM scm_makfromstrs ();
-extern SCM scm_take0str ();
-extern SCM scm_makfromstr ();
-extern SCM makfrom0str ();
-extern SCM makfrom0str_opt ();
-extern SCM scm_string_p ();
-extern SCM scm_string ();
-extern SCM scm_make_string ();
-extern SCM scm_string_length ();
-extern SCM scm_string_ref ();
-extern SCM scm_string_set ();
-extern SCM scm_string_equal_p ();
-extern SCM scm_string_ci_equal_p ();
-extern SCM scm_string_less_p ();
-extern SCM scm_string_leq_p ();
-extern SCM scm_string_gr_p ();
-extern SCM scm_string_geq_p ();
-extern SCM scm_string_ci_less_p ();
-extern SCM scm_string_ci_leq_p ();
-extern SCM scm_string_ci_gr_p ();
-extern SCM scm_string_ci_geq_p ();
-extern SCM scm_substring ();
-extern SCM scm_string_append ();
-extern void scm_init_strings ();
-
-#endif /* STDC */
-
-
-
-
-
-
-
-
-
-
 
+extern SCM scm_string_p SCM_P ((SCM x));
+extern SCM scm_read_only_string_p SCM_P ((SCM x));
+extern SCM scm_string SCM_P ((SCM chrs));
+extern SCM scm_makstr SCM_P ((long len, int slots));
+extern SCM scm_makfromstrs SCM_P ((int argc, char **argv));
+extern SCM scm_take0str SCM_P ((char * it));
+extern SCM scm_makfromstr SCM_P ((const char *src, scm_sizet len, int slots));
+extern SCM scm_makfrom0str SCM_P ((const char *src));
+extern SCM scm_makfrom0str_opt SCM_P ((const char *src));
+extern SCM scm_make_string SCM_P ((SCM k, SCM chr));
+extern SCM scm_string_length SCM_P ((SCM str));
+extern SCM scm_string_ref SCM_P ((SCM str, SCM k));
+extern SCM scm_string_set_x SCM_P ((SCM str, SCM k, SCM chr));
+extern SCM scm_substring SCM_P ((SCM str, SCM start, SCM end));
+extern SCM scm_string_append SCM_P ((SCM args));
+extern SCM scm_make_shared_substring SCM_P ((SCM str, SCM frm, SCM to));
+extern void scm_init_strings SCM_P ((void));
 
 #endif  /* STRINGSH */
