@@ -2,7 +2,7 @@
 
 #ifndef THROWH
 #define THROWH
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,31 +44,43 @@
  */
 
 
-#include "__scm.h"
+#include "libguile/__scm.h"
 
 
 
-extern SCM scm_bad_throw_vcell;
+typedef SCM (*scm_catch_body_t) SCM_P ((void *data, SCM jmpbuf));
+typedef SCM (*scm_catch_handler_t) SCM_P ((void *data,
+					   SCM tag, SCM throw_args));
 
-
-#ifdef __STDC__
-extern SCM scm_catch (SCM tag, SCM thunk, SCM handler);
-extern SCM _scm_throw (SCM key, SCM args, int noreturn);
-extern SCM scm_throw_exception (SCM key, SCM args);
-extern SCM scm_dynamic_root (void);
-extern void scm_init_throw (void);
-
-#else /* STDC */
-extern SCM scm_catch ();
-extern SCM _scm_throw ();
-extern SCM scm_throw_exception ();
-extern SCM scm_dynamic_root ();
-extern void scm_init_throw ();
-
-#endif /* STDC */
+extern SCM scm_internal_catch SCM_P ((SCM tag,
+				      scm_catch_body_t body,
+				      void *body_data,
+				      scm_catch_handler_t handler,
+				      void *handler_data));
 
+/* The first argument to scm_body_thunk should be a pointer to one of
+   these.  See the implementation of catch in throw.c.  */
+struct scm_body_thunk_data
+{
+  /* The tag being caught.  We only use it to figure out what
+     arguments to pass to the body procedure; see scm_catch_thunk_body for
+     details.  */
+  SCM tag;
 
+  /* The Scheme procedure object constituting the catch body.
+     scm_body_by_proc invokes this.  */
+  SCM body_proc;
+};
+
+extern SCM scm_body_thunk SCM_P ((void *, SCM));
 
 
+extern SCM scm_handle_by_proc SCM_P ((void *, SCM, SCM));
+extern SCM scm_handle_by_message SCM_P ((void *, SCM, SCM));
 
+extern SCM scm_catch SCM_P ((SCM tag, SCM thunk, SCM handler));
+extern SCM scm_lazy_catch SCM_P ((SCM tag, SCM thunk, SCM handler));
+extern SCM scm_ithrow SCM_P ((SCM key, SCM args, int noreturn));
+extern SCM scm_throw SCM_P ((SCM key, SCM args));
+extern void scm_init_throw SCM_P ((void));
 #endif  /* THROWH */
